# Generated by roxygen2: do not edit by hand

S3method(add_finalizer,BatchtoolsFuture)
S3method(delete,BatchtoolsFuture)
S3method(interruptFuture,BatchtoolsFutureBackend)
S3method(launchFuture,BatchtoolsFutureBackend)
S3method(loggedError,BatchtoolsFuture)
S3method(loggedOutput,BatchtoolsFuture)
S3method(nbrOfFreeWorkers,BatchtoolsFutureBackend)
S3method(nbrOfFreeWorkers,BatchtoolsMultiprocessFutureBackend)
S3method(nbrOfFreeWorkers,BatchtoolsUniprocessFutureBackend)
S3method(nbrOfWorkers,BatchtoolsFutureBackend)
S3method(nbrOfWorkers,BatchtoolsMulticoreFutureBackend)
S3method(nbrOfWorkers,BatchtoolsUniprocessFutureBackend)
S3method(print,BatchtoolsFuture)
S3method(print,BatchtoolsFutureBackend)
S3method(print,BatchtoolsSGEFutureBackend)
S3method(print,BatchtoolsSlurmFutureBackend)
S3method(registerFuture,BatchtoolsFuture)
S3method(registerFuture,BatchtoolsUniprocessFuture)
S3method(registerFuture,default)
S3method(resolved,BatchtoolsFuture)
S3method(result,BatchtoolsFuture)
S3method(stopWorkers,BatchtoolsFutureBackend)
S3method(unregisterFuture,BatchtoolsFuture)
S3method(unregisterFuture,BatchtoolsUniprocessFuture)
S3method(unregisterFuture,default)
S3method(waitForWorker,BatchtoolsFuture)
S3method(waitForWorker,BatchtoolsUniprocessFuture)
S3method(waitForWorker,default)
export("%resources%")
export(BatchtoolsBashFutureBackend)
export(BatchtoolsCustomFutureBackend)
export(BatchtoolsFutureBackend)
export(BatchtoolsFutureError)
export(BatchtoolsInteractiveFutureBackend)
export(BatchtoolsLocalFutureBackend)
export(BatchtoolsLsfFutureBackend)
export(BatchtoolsMulticoreFutureBackend)
export(BatchtoolsMultiprocessFutureBackend)
export(BatchtoolsOpenLavaFutureBackend)
export(BatchtoolsSGEFutureBackend)
export(BatchtoolsSSHFutureBackend)
export(BatchtoolsSlurmFutureBackend)
export(BatchtoolsTemplateFutureBackend)
export(BatchtoolsTorqueFutureBackend)
export(BatchtoolsUniprocessFutureBackend)
export(batchtools_bash)
export(batchtools_custom)
export(batchtools_interactive)
export(batchtools_local)
export(batchtools_lsf)
export(batchtools_multicore)
export(batchtools_openlava)
export(batchtools_sge)
export(batchtools_slurm)
export(batchtools_ssh)
export(batchtools_torque)
export(loggedError)
export(loggedOutput)
export(makeClusterFunctionsBash)
export(makeClusterFunctionsSlurm2)
importFrom(batchtools,Worker)
importFrom(batchtools,assertRegistry)
importFrom(batchtools,batchExport)
importFrom(batchtools,batchMap)
importFrom(batchtools,cfBrewTemplate)
importFrom(batchtools,cfReadBrewTemplate)
importFrom(batchtools,clearRegistry)
importFrom(batchtools,findConfFile)
importFrom(batchtools,findTemplateFile)
importFrom(batchtools,getErrorMessages)
importFrom(batchtools,getLog)
importFrom(batchtools,getStatus)
importFrom(batchtools,killJobs)
importFrom(batchtools,loadResult)
importFrom(batchtools,makeClusterFunctions)
importFrom(batchtools,makeClusterFunctionsInteractive)
importFrom(batchtools,makeClusterFunctionsLSF)
importFrom(batchtools,makeClusterFunctionsMulticore)
importFrom(batchtools,makeClusterFunctionsOpenLava)
importFrom(batchtools,makeClusterFunctionsSGE)
importFrom(batchtools,makeClusterFunctionsSSH)
importFrom(batchtools,makeClusterFunctionsSlurm)
importFrom(batchtools,makeClusterFunctionsTORQUE)
importFrom(batchtools,makeRegistry)
importFrom(batchtools,makeSubmitJobResult)
importFrom(batchtools,removeRegistry)
importFrom(batchtools,saveRegistry)
importFrom(batchtools,setJobNames)
importFrom(batchtools,submitJobs)
importFrom(batchtools,waitForJobs)
importFrom(checkmate,"%??%")
importFrom(checkmate,assertCharacter)
importFrom(checkmate,assertString)
importFrom(future,FutureBackend)
importFrom(future,FutureError)
importFrom(future,FutureInterruptError)
importFrom(future,FutureLaunchError)
importFrom(future,FutureWarning)
importFrom(future,cancel)
importFrom(future,interruptFuture)
importFrom(future,launchFuture)
importFrom(future,nbrOfFreeWorkers)
importFrom(future,nbrOfWorkers)
importFrom(future,plan)
importFrom(future,resolved)
importFrom(future,result)
importFrom(future,run)
importFrom(future,stopWorkers)
importFrom(future,tweak)
importFrom(parallelly,availableCores)
importFrom(parallelly,availableWorkers)
importFrom(parallelly,supportsMulticore)
importFrom(stringi,stri_flatten)
importFrom(stringi,stri_replace_all_fixed)
importFrom(tools,pskill)
importFrom(utils,capture.output)
importFrom(utils,file_test)
importFrom(utils,packageVersion)
importFrom(utils,sessionInfo)
importFrom(utils,str)
importFrom(utils,tail)
