## Look for detritus files
testme <- as.environment("testme")
delta <- diff_connections(get_connections(), testme[["testme_connections"]])
if (any(lengths(delta) > 0)) {
  message(sprintf("Detritus connections generated by test %s:", sQuote(testme[["name"]])))
  print(delta)
}

