#! /usr/bin/env Rscript
## This runs testme test script inst/testme/test-zzz,future_lapply.R
## Don't edit - it was autogenerated by inst/testme/deploy.R
future.batchtools:::testme("zzz,future_lapply")
